/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>
#include <time.h>

#include "rubrica.h"
#include "modulo.h"
#include "main.h"
#include "file.h"
#include "file_util.h"
#include "prefer.h"
#include "dictionary.h"
#include "addressbook.h"

VistaDati vista;

Dictionary dict;
gchar *language;
gboolean language_tested = FALSE;
gboolean recover = FALSE;

gchar *nomefile;

gchar *rubrica_version;
gchar *current_version = VERSION;


void rubrica_file_init_names(void)
{
  if (nomefile)
    g_free(nomefile);

  nomefile = NULL;
}


void rubrica_file_set_filename(gchar *filename)
{
  if (nomefile)
    g_free(nomefile);

  nomefile = g_strdup(filename);   
}


void rubrica_file_salva(void)
{
  GtkWidget *app;
  gchar *tmp;

  app = rubrica_get_app();
  /* 
     se non c' un nome file di default allora bisogna richiamare salva_con_nome     
     if nomefile is NULL, then call rubrica_file_salva_come (ie save as)
  */
  if (!nomefile || g_strcasecmp(nomefile, _("no name")) == 0)
    /* salvo come default? */
    rubrica_file_salva_come(NULL, NULL);

    /*   altrimenti provo a scrivere il file 
    */
  else 
    if (!rubrica_file_write_xmlfile(nomefile))
      {
	tmp = g_strdup_printf(_("I can't save the file:\n%s"),nomefile);
	gnome_app_error(GNOME_APP(app), tmp);
	g_free(tmp);
      }
}

void 
rubrica_file_salva_come(GtkWidget *w, gpointer data)
{
  GtkWidget *app;
  GtkFileSelection *fs;
  
  app = rubrica_get_app();
  
  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Rubrica Save as")));

  if (rubrica_preferences_get_have_default_folder())
    {
      /* TODO 
	 
         settate la directory di default come corrente 
      */
    }
  
  /* se ho gi un nome di file lo mostro nell'entry */
  if (nomefile)
    gtk_file_selection_set_filename(fs, nomefile);
  
  /* connetto i bottoni ok e cancella */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_file_scrivi_file), fs);
  
  /* con il bottone cancella distruggo direttamente il widget */
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));
  
  gtk_widget_show(GTK_WIDGET(fs));
}


void
rubrica_file_scrivi_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *tmp;
  gchar *fname=NULL;
  Rubrica *rub;
  gint page;      /* notebook's page, now active */

  page = rubrica_addressbook_get_current_page();
  rub = rubrica_addressbook_get_rubrica(page);

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));
  g_return_if_fail(rub != NULL);

  tmp = gtk_file_selection_get_filename(fs);

  /* se il file non ha estensione o l'estensione non  .rub */
  if (!rubrica_file_has_extension(tmp))
    fname = g_strdup_printf("%s.rub",tmp);  
  else 
    fname = g_strdup(tmp);
  
  /* se devo effettuare il backup */
  if (rubrica_preferences_get_do_backup())
    {
      gchar *newname;

      newname = g_strdup_printf("%s1", fname);
      rubrica_file_do_backup(fname, newname);
      g_free(newname);
    }

  /* se non  stato selezionato nessun file
     o non  possibile aprire il file segnalare l'errore */  
  if (!fname || !rubrica_file_write_xmlfile(fname)) 
    {
      gchar *err;
      GtkWidget *dlg;
      
      /* il dialogo di errore ha il widget fs come genitore */
      if (fname)
	err = g_strdup_printf(_("I can't save the file %s"), fname); 
      else
	err = g_strdup_printf(_("I can't save the file")); 
      
      dlg = gnome_error_dialog_parented(err, GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);   
      g_free(err);
    }
  else /* il file  stato salvato */
    {      
      rubrica_addressbook_set_flags(FALSE, FALSE, FALSE, page);
      rubrica_addressbook_set_path_and_name(g_dirname(fname), g_basename(fname), page);
      
      /* imposto i nuovi filename */
      rubrica_file_set_filename(g_basename(fname));
      
      /* imposta il file appena salvato come l'ultimo aperto/visualizzato 
      */
      rubrica_preferences_set_last_book(fname, RUBRICA_KEY_LAST_BOOK);
      
      rubrica_addressbook_make_current(rub);

      gtk_widget_destroy(GTK_WIDGET(fs));
    }

  g_free(fname);
}

gboolean
rubrica_file_write_xmlfile(gchar *name)
{
  xmlDocPtr doc;  /* puntatore al documento xml */
  GList *rub;     /* ptr to rubrica, this contains data that we want save */
  GList *l;       /* alias to rub */
  gchar *path, *fname;

  rub = rubrica_get_rubrica();

  g_return_val_if_fail(name != NULL, FALSE);
  g_return_val_if_fail(rub != NULL, FALSE);
  
  /* se l'utente vuole salvare le rubriche in una cartella predefinita 
     if users want to save addressbook in a default folder
  */
  if (rubrica_preferences_get_have_default_folder())  
    {
      name = g_basename(name);
      path = rubrica_preferences_get_default_folder();
      fname = g_strdup_printf("%s/%s", path, name);
    }
  else
    fname = g_strdup(name);

  /* creazione di un nuovo documento xml */
  doc = xmlNewDoc("1.0");
  
  /* creo un nuovo nodo radice rubrica */
  doc->root = xmlNewDocNode(doc, NULL, "Rubrica", NULL);
  xmlSetProp(doc->root, "version", current_version);

  /* attraverso la rubrica e scrivo i dati 
     nell'albero xml
  */
  for (l = rub; l != NULL; l = l->next) 
    {
      RubricaItem *item = l->data;
      
      rubrica_file_build_xmlnode(doc->root, item);
    }
  
  /* verifico che non esista un file con lo stesso nome dell'attuale
     se chiesto, faccio il backup del vecchio
     altrimenti cancello il vecchio file e scrivo il nuovo 
  */
  if (g_file_exists(fname))
    {
      if (rubrica_preferences_get_do_backup())
	rubrica_file_do_backup(fname, fname);
      else
	remove(fname);
    }
  
  if (rubrica_preferences_get_compress)
    {
      xmlSetCompressMode(rubrica_preferences_get_compress_rate());
      xmlSetDocCompressMode(doc, rubrica_preferences_get_compress_rate());
    }
  else
    {
      xmlSetDocCompressMode(doc, 0);     
      xmlSetCompressMode(0);
    }
  
  if (xmlSaveFile(fname, doc) == -1) 
    {
      xmlFreeDoc(doc);
      return FALSE;
    }  

  /* if we are here the addressbook is write */
  g_print("\nfile saved as %s in %s", name, path);

  /* xmlFreeDoc do a segfault, why? */
  //  xmlFreeDoc(doc);    
  g_free(fname);

  return TRUE;
}

void 
rubrica_file_build_xmlnode(xmlNodePtr xmlnode, RubricaItem *item)
{
  GList *alias;
  xmlNodePtr newxml;

  xmlNodePtr schedaxml;
  xmlNodePtr datixml;
  xmlNodePtr webxml;
  xmlNodePtr mailxml;
  xmlNodePtr telxml;
  xmlNodePtr notexml;
  gchar *str;

  xmlNodePtr nodo;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(item != NULL);
  
  /*
    newxml  figlio di xmlnode (il primo xmlnode  doc->root)
    boolxml, schedaxml, datixml, ..., sono figli di newxml
  */
    
  newxml = xmlNewChild(xmlnode, NULL, "Address", NULL);
  str = g_strdup_printf("%ld", item->created);
  xmlSetProp(newxml, "created", str);
  g_free(str);
  str = g_strdup_printf("%ld", item->last_change);  
  xmlSetProp(newxml, "last_change", str);
  g_free(str);
  
  /*
    xmlNewTextChild 
    sintassi :
         - nodo genitore del nodo creato con la chiamata
	 - namespace
	 - nome del nodo creato
	 - testo (elemento) da inserire nel nodo creato
  */
  schedaxml = xmlNewTextChild(newxml, NULL, "Card", item->scheda);

  /* setto le propriet della scheda (editabile, concellabile) */
  if (item->editabile)
    xmlSetProp(schedaxml, "modifying", "TRUE");
  else
    xmlSetProp(schedaxml, "modifying", "FALSE");  

  if (item->cancellabile)
    xmlSetProp(schedaxml, "deleting", "TRUE");
  else
    xmlSetProp(schedaxml, "deleting", "FALSE");
  
  /* ho creato l'elemento xml es.: 
     <Scheda editabile="TRUE" cancellabile="FALSE>scheda numero uno</Scheda>
     <Card modifying="TRUE" deleting="FALSE">card number one</Card>
  */

  datixml = xmlNewTextChild(newxml, NULL, "Data", NULL);
  nodo = xmlNewTextChild(datixml, NULL, "FirstName", item->nome);
//  nodo = xmlNewTextChild(datixml, NULL, "MiddleName", item->secondo_nome);
  nodo = xmlNewTextChild(datixml, NULL, "LastName", item->cognome);
  nodo = xmlNewTextChild(datixml, NULL, "Profession", item->professione);
  nodo = xmlNewTextChild(datixml, NULL, "Street", item->via);
  nodo = xmlNewTextChild(datixml, NULL, "StreetNumber", item->numero);
  nodo = xmlNewTextChild(datixml, NULL, "ZipCode", item->cap);
  nodo = xmlNewTextChild(datixml, NULL, "City", item->citta);
  nodo = xmlNewTextChild(datixml, NULL, "Province", item->provincia);
  nodo = xmlNewTextChild(datixml, NULL, "Country", item->stato);
  
  webxml = xmlNewChild(newxml, NULL, "WebAddresses", NULL);
  alias = item->web_list;
  while(alias)
    {
      nodo = xmlNewTextChild(webxml, NULL, "Web", alias->data);

      alias = alias->next;
    }
  
  mailxml = xmlNewChild(newxml, NULL, "EmailAddresses", NULL);
  alias = item->mail_list;
  while(alias)
    {
      nodo = xmlNewTextChild(mailxml, NULL, "Email", alias->data);
  
      alias = alias->next;
    }

  telxml = xmlNewChild(newxml, NULL, "TelephoneNumbers", NULL);
  alias = item->tel_list;
  while(alias)
    {
      TelNum *tt = (TelNum*) alias->data;

      nodo = xmlNewTextChild(telxml, NULL, "Telephone", tt->numero_telefono);      
      xmlSetProp(nodo, "type", tt->tipo);
      
      alias = alias->next;
    }
  
  notexml = xmlNewTextChild(newxml, NULL, "Notes", item->note);
}



/*      Load Routines
*/
/*  this function is called when user want start rubrica whit a file
*/
void rubrica_file_start_with_file(gchar *file)
{
  Rubrica *rub;
  gboolean loaded;
  
  rubrica_file_set_start(TRUE);

  /* load file */
  loaded = rubrica_file_load_xmlfile(file);
  if (!loaded)
    {
      rubrica_error_generic(_("I can't load default addressbook."));
    }

  rubrica_file_set_start(FALSE);
  rub = rubrica_addressbook_get_rubrica(0);
  rubrica_addressbook_make_current(rub);
}

gboolean rubrica_file_load_xmlfile(gchar *filename)
{
  gint page;
  gchar *fpath, *fname;
  gboolean user_dict;  

  if (!g_file_exists(filename))
    {
      gchar *str = g_strdup_printf(_("I can't read %s.\nFile don't exist."),filename);
      
      rubrica_error_generic(str);
            
      g_free(str);
      return FALSE;
    }

  /* if user want to use his dictionary */
  user_dict = rubrica_preferences_get_user_has_dictionary();

  if (user_dict)
    {
      /* if dictionary isn't loaded, then load it */
      if(!dictionary_loaded())
	rubrica_dictionary_load_translation();  
      
      /* get dictionary (from memory, only choose one dictionary)

	 NULL - load other (user) dictionary
	 IT   - load italian dictionary
	 EN   - load english dictionary
       */
      dict = rubrica_dictionary_get_dictionary(NULL);
    }

  /* real file load */
  if (!rubrica_file_real_load_xmlfile(filename)) 
    return FALSE;
  else
    /* file has been loaded 
     */
    rubrica_set_empty(FALSE);      
  
  /* if we are here, then the file has been loaded, now we need 
     to put in memory some information about this file and addressbook      
  */
  page = rubrica_addressbook_get_current_page();

  fpath = g_dirname(filename);
  fname = g_basename(filename);

  rubrica_addressbook_set_flags(FALSE, FALSE,FALSE, page);
  rubrica_addressbook_set_path_and_name(fpath, fname, page); 

  g_free(fpath);

  return TRUE;
}


void 
rubrica_file_carica_file(void)
{
  GtkFileSelection *fs;
  GtkWidget *app;

  app = rubrica_get_app();

  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Open file...")));
  
  /* connetto i bottoni ok e cancel */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_file_apri_file), fs);
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  /* visualizzo la finestra in posizione centrale */
  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

  /* il genitore della finestra fileselection  app */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  gtk_widget_show(GTK_WIDGET(fs));
}

void 
rubrica_file_apri_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *fname;
  gchar *tmp;
  gboolean loaded;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  fname = gtk_file_selection_get_filename(fs);

  loaded = rubrica_file_load_xmlfile(fname);
  if (!loaded)
    {
      GtkWidget *dlg;
      
      tmp = g_strdup_printf(_("I can't read this file:\n %s"), fname);
      
      dlg = gnome_error_dialog_parented(tmp,GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);

      g_free(tmp);
    }
  else
    gtk_widget_destroy(GTK_WIDGET(fs));
}


/* 
 *  read the addressbook file 
 */
gboolean 
rubrica_file_real_load_xmlfile(gchar *fname)
{
  xmlDocPtr doc;
  gchar *lang; /* language set in file */

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* read and parse file */
  doc = xmlParseFile(fname);

  if (!doc)
    return FALSE;

  if (!doc->root || 
      !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      xmlFreeDoc(doc);
      return FALSE;
    }
  lang = xmlNodeGetLang(doc->root);

  if(!lang)
    {
      /* may be this file has been created with an rubrica version less then 0.5
	 Those version use english as language
      */
      recover = TRUE;
    }
  else
    {
      if (strcmp(lang, "IT") == 0)
	dict = rubrica_dictionary_get_dictionary("IT");
      else     
	if (strcmp(lang, "EN") == 0)
	  dict = rubrica_dictionary_get_dictionary("EN");
	else
	  dict = rubrica_dictionary_get_dictionary(NULL);
    }

  /* parse doc, extract address from file and put them in memory */
  rubrica_file_parse_doc(doc, fname);

  /* addressbook has been parsed. Free memory */
  xmlFreeDoc(doc);
 
  rubrica_file_set_filename(g_basename(fname)); 
 
  return TRUE;  
}


void 
rubrica_file_parse_doc(xmlDocPtr doc, gchar *fname)
{  
  xmlNodePtr nodo;
  gint page = 0;

  /* inizializzo le strutture necessarie per 
     caricare una nuova lista
  */
  if (!rubrica_file_is_start())
    page = rubrica_addressbook_add_page(fname);
     
  /* try to load an old or damaged file
   */
  if(recover)
    {  
      xmlNodePtr xmltmp;

      xmltmp = doc->root->childs;
      if (g_strcasecmp(xmltmp->name, "Address") == 0)
	dict = rubrica_dictionary_get_dictionary("EN");
      if (g_strcasecmp(xmltmp->name, "Indirizzo") == 0)
	dict = rubrica_dictionary_get_dictionary("IT");
      else
	{	/* procedura disperata, sottoponi all'utente il file */ 

	}
    }

  if ((doc->root->properties)       &&
      (doc->root->properties->name) &&
      strcmp(doc->root->properties->name, "version") == 0)
    rubrica_version = xmlGetProp(doc->root, "version");
  else
    rubrica_version = NULL;

  /* prendo il nodo Indirizzo dal file e lo inserisco nella (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)    
    rubrica_file_load_from_xmlnode(nodo, NULL);  
}

void
rubrica_file_load_from_xmlnode(xmlNodePtr xmlnode, GList *l)
{
  RubricaItem *r;  
  gboolean load_rescue = FALSE;
  gchar *buffer;
  GList *web, *mail, *tel;
  xmlNodePtr figlio;
  /* i nodi xmlnode sono nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       schedaxml, ...
  */

  if (!xmlnode->name ||                               /* se il nodo non ha nome o */
      g_strcasecmp(xmlnode->name, dict.address) != 0) /* il nome non  Indirizzo  */
    {
      /* We found an error. We have loaded onje dictionary, but 
	 xmlnode->name and dict.address don't match. 
	 Will do:
	 - Try other dictionary 
	 - if fail try to recover file with user help
      */
      if (xmlnode->name)
	{
	  gboolean dict_found = FALSE;

	  /* try italian dictionary */
	  dict = rubrica_dictionary_get_dictionary("IT");
	  if (g_strcasecmp(xmlnode->name, dict.address) == 0)
	    dict_found = TRUE;	  /* OK found */
	  else
	    {
	      /* try english dictionary */
	      dict = rubrica_dictionary_get_dictionary("EN");	      
	      if (g_strcasecmp(xmlnode->name, dict.address) == 0)
		dict_found = TRUE;  /* OK found */
	      else
		{
		  /* try user dictionary */
		  dict = rubrica_dictionary_get_dictionary(NULL);	      
		  if (g_strcasecmp(xmlnode->name, dict.address) == 0)
		    dict_found = TRUE;	  /* OK found */
		}
	    }
	  if (!dict_found)	    
	    load_rescue = TRUE;	    
	}      
    }
   
  r = g_malloc(sizeof(RubricaItem));

  if ((rubrica_version != NULL) &&
      (strcmp(rubrica_version, current_version) == 0))
    {
      gchar *tmp;
      
      tmp = xmlGetProp(xmlnode, "created");
      r->created = (tmp != NULL ? atol(tmp) : time(NULL));
      g_free(tmp);
      
      tmp = xmlGetProp(xmlnode, "last_change");
      r->last_change = (tmp != NULL ? atol(tmp) : time(NULL));
      g_free(tmp);
    }
  else
    {
      r->last_change = r->created = time(NULL);
    }

  /* schedaxml */
  figlio = xmlnode->childs;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.card) != 0)    
    r->scheda = "";    
  else     
    r->scheda = g_strdup(xmlNodeGetContent(figlio));
     
  /* scheda ha degli attributi, bisogna estrarli */
  buffer = xmlGetProp(figlio, dict.modifying);
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->editabile = TRUE;
  else 
    r->editabile = FALSE;
  
  buffer = xmlGetProp(figlio, dict.deleting);
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->cancellabile = TRUE;
  else 
    r->cancellabile = FALSE;  
  
  /* dati personali -- datixml -- */
  figlio = figlio->next; 
  if (!figlio->name || g_strcasecmp(figlio->name, dict.data) != 0)
    r->nome = r->cognome 
      = r->professione = r->via 
      = r->numero = r->cap 
      = r->citta = r->provincia 
      = r->stato = "";
  else 
    {
      /* carico i figli di datixml
	 - datixml=figlio 
	 - nodo = figlio di datixml
      */
      xmlNodePtr nodo;
      gchar *tmp;
      
      nodo = figlio->childs;
      tmp = xmlNodeGetContent(nodo);
      r->nome = (tmp ? g_strdup(tmp) : "");
    
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->cognome = (tmp ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->professione = (tmp ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->via = (tmp ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->numero  = (tmp ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->cap = (tmp ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->citta = (tmp ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->provincia = (tmp ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      r->stato = (tmp ? g_strdup(tmp) : "");
    }
  
  /* web */
  web = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.webaddresses) != 0)
    {
      r->web_list = NULL;
      r->nweb = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;
	  
	  tmp = xmlNodeGetContent(nodo);
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      web = g_list_append(web, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->web_list = web;
      r->nweb = n;
    }
  
  /* email */
  figlio = figlio->next;
  mail = NULL;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.mailaddresses) != 0)
    {
      r->mail_list = NULL;
      r->nmail = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;
	  
	  tmp = xmlNodeGetContent(nodo);
	  
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      mail = g_list_append(mail, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->mail_list = mail;
      r->nmail = n;
    }  
  
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.tellist) != 0)
    {
      r->tel_list = NULL;
      r->ntel = 0;
    }
  else
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp1=NULL, *tmp2=NULL;
	  
	  tmp1 = xmlNodeGetContent(nodo);
	  if ((strcmp(nodo->properties->name, "Type") == 0))
	    tmp2 = xmlGetProp(nodo, "Type");
	  else
	    tmp2 = xmlGetProp(nodo, dict.teltype);

	  if (tmp1)
	    {	
	      t = g_malloc(sizeof(TelNum));
	      
	      t->numero_telefono = g_strdup(tmp1);
	      t->tipo = g_strdup(tmp2);	  	     
	      
	      tel = g_list_append(tel, t);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->tel_list = tel;
      r->ntel = n;
    }
  
  /* note */
  figlio = figlio->next;
  r->note = NULL;
  if (figlio->name && g_strcasecmp(figlio->name, dict.note) == 0)
    {
      gchar *tmp;
      
      tmp = xmlNodeGetContent(figlio);
      
      if (tmp)
	r->note = g_strdup(tmp);
    } 

  rubrica_inserisci_elemento(r);
  rubrica_view_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}



