/*
*  Rubrica
*  file: import_gcard.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "rubrica.h"
#include "import.h"
#include "import_gcard.h"

RubricaItem *r;
TelNum *tt;
GList *web=NULL, *mail=NULL, *tel=NULL;
GList *notelst, *gcnotelst;
gint notelen, gcnotelen;

gint ntel, nmail;
gboolean morerow = FALSE;

GToken gtoken[] = { 
  {"BEGIN", BEGIN, FALSE},            {"FN", FN, FALSE},  
  {"N", N, FALSE},                    {"REV", REV, FALSE},  
  {"BDAY", BDAY, FALSE},              {"ADR", ADR, TRUE},
  {"TEL", TEL, TRUE},                 {"EMAIL", EMAIL, TRUE},  
  {"URL", URL, FALSE},                {"TZ", TZ, FALSE},
  {"GEO",GEO},                        {"TITLE", TITLE, FALSE},            
  {"ORG", ORG, FALSE},                {"CATEGORIES", CATEGORIES, FALSE},  
  {"NOTE", NOTE, FALSE},              {"KEY", KEY, TRUE}, 
  {"END", END, FALSE},                {"BAD_GTOKEN", BAD, FALSE}
};

SubGtoken subgtoken[] = {
  {"HOME", HOME},         {"WORK",WORK },             {"POSTAL",POSTAL}, 
  {"PARCEL",PARCEL },     {"DOM", DOM},               {"INTL",INTL },   
  {"PREF",PREF },         {"VOICE",VOICE },           {"FAX", FAX},
  {"MSG",MSG },           {"CELL",CELL },             {"PAGER",PAGER },
  {"BBS", BBS},           {"MODEM",MODEM },           {"CAR",CAR },      
  {"ISDN", ISDN},         {"VIDEO",VIDEO},            {"AOL",AOL }, 
  {"INTERNET",INTERNET }, {"APPLELINK",APPLELINK },   {"ATTMAIL", ATTMAIL}, 
  {"CIS", CIS},           {"EWORLD", EWORLD},         {"IBMMail",IBMMail },
  {"MCIMAIL",MCIMAIL },   {"POWERSHARE",POWERSHARE }, {"PRODIGY", PRODIGY},
  {"TLX", TLX},           {"X400",X400},              {"VCARD", VCARD},
  {"PGP",PGP},            {"X509",X509},
  {"QUOTED-PRINTABLE",QUOTED_PRINTABLE},              {"BAD_SUBGTOKEN", SUB_END}
};		


void rubrica_import_gcard_decode_buffer(gchar *buffer)
{
  gchar *str=NULL, *data=NULL, *tmp=NULL;
  gchar *decoded[8];

  TokenGCard token;
  SubGToken subtoken;
  gint datalen = 0;

  g_return_if_fail(buffer != NULL);
    
  rubrica_import_gcard_split_buffer(buffer, &str, &data, &morerow); 
  token = rubrica_import_gcard_decode_token(str, &subtoken);

  switch(token)
    {
    case BEGIN:
      /* dispose memory for new address */
      r = (RubricaItem*) g_malloc(sizeof(RubricaItem));
      ntel = nmail = notelen = gcnotelen = 0;
      notelst = NULL;
      gcnotelst = NULL;
      rubrica_import_init_item(r);
      morerow = FALSE;
      break;
      
      /*	case VCARD: do nothing 
      */

    case FN:
      /* card name */ 
      r->scheda = g_strdup(data);      
      g_free(data);
      morerow = FALSE;
      break;
      
    case N:
      /* ord:: surname, second name, prefix, suffix 
	       cognome, nome, secondo nome, prefisso, suffisso */
      rubrica_import_gcard_decode_string(data, decoded);
 
      r->cognome = g_strdup(decoded[0]);
      r->nome = g_strdup(decoded[1]);
      /*
	decoded[2] <= second name
	decoded[3] <= prefix
	decoded[4] <= suffix
       */
      if (decoded[2])
	{
	  tmp = g_strdup_printf(_("Second name: %s\n"),decoded[2]);
	  notelen += strlen(tmp);
	  notelst = g_list_append(notelst, tmp);
	}
      if (decoded[3])
	{
	  tmp = g_strdup_printf(_("Prefix: %s\n"),decoded[3]);
	  notelen += strlen(tmp);
	  notelst = g_list_append(notelst, tmp);     
	}
      if (decoded[4])
	{
	  tmp = g_strdup_printf(_("Suffix: %s\n"),decoded[4]);
	  notelen += strlen(tmp);
	  notelst = g_list_append(notelst, tmp);      
	}

      g_free(data);
      morerow = FALSE;
      break;

    case REV:
      /* date, fuso orario, time
	 data, fuso orario, ora */
      tmp = g_strdup_printf(_("Fuso orario: %s\n"),data);
      notelen += strlen(tmp);
      notelst = g_list_append(notelst, tmp);
      
      g_free(data);
      morerow = FALSE;
      break;

    case BDAY:
      tmp = g_strdup_printf(_("Birthday: %s\n"),data);
      notelen += strlen(tmp);
      notelst = g_list_append(notelst, tmp);

      g_free(data);
      morerow = FALSE;
      break;

    case ADR:
      /* mail address + mail type 
	 indirizzo di posta + tipo */

      rubrica_import_gcard_decode_string(data, decoded);

      r->via = g_strdup(decoded[2]);
      r->numero = g_strdup("");
      r->cap = g_strdup(decoded[5]);
      r->citta = g_strdup(decoded[3]);
      r->provincia = g_strdup(decoded[4]);
      r->stato = g_strdup(decoded[6]);
      /* 
	 decoded[0] <= post office
	 decoded[1] <= extended
	 decoded[4] <= region ==> province
      */
      if (decoded[0])
	{
	  tmp = g_strdup_printf(_("Post Office: %s\n"),decoded[0]);
	  notelen += strlen(tmp);
	  notelst = g_list_append(notelst, tmp);      
	}
      if (decoded[1])
	{
	  tmp = g_strdup_printf(_("Extended: %s\n"),decoded[1]);
	  notelen += strlen(tmp);
	  notelst = g_list_append(notelst, tmp);      
	}
      morerow = FALSE;
      g_free(data);
      break;

    case TEL:
      /* telefono + tipo */
      tt = (TelNum*) g_malloc(sizeof(TelNum));
      tt->numero_telefono = g_strdup(data);

      switch (subtoken)
	{	  
	case HOME:
	  tt->tipo = "Casa";
	  break;
	case WORK:
	  tt->tipo = "Ufficio";
	  break;
	case FAX:
	  tt->tipo = "Fax";
	  break;
	case CELL:
	  tt->tipo = "Cellulare";
	  break;
	  
	  /* altro */
	case PREF:
	case VOICE:
	case MSG:
	case PAGER:
	case BBS:
	case MODEM:
	case CAR:
	case ISDN:
	case VIDEO:
	default:
	  tt->tipo = "Altro";
	  break;
	} 

      tel = g_list_append(tel, tt);	  
      ntel++;     

      g_free(data);
      morerow = FALSE;
      break;
      
    case EMAIL:      
      /* email +tipo */
      switch(subtoken)
	{
	case AOL:
	case INTERNET:
	case APPLELINK:
	case ATTMAIL:
	case CIS:
	case EWORLD:
	case IBMMail:
	case MCIMAIL:
	case POWERSHARE:
	case PRODIGY:
	case TLX:
	case X400:
	default:
	  tmp = g_strdup(data);
	  if (tmp)
	    {
	      mail = g_list_append(mail, tmp);	  
	      nmail++;
	    }
	  break;
	}  /* switch subtoken */
      g_free(data);
      morerow = FALSE;
      break;

      /* url */
    case URL:      
      tmp = g_strdup(data);
      web = g_list_append(web, tmp);      
      r->web_list = web;
      r->nweb = 1;

      g_free(data);
      morerow = FALSE;
      break;

    case TZ:
      /* Time Zone */
      tmp = g_strdup_printf("Time Zone: %s\n", data);
      notelen += strlen(tmp);
      notelst = g_list_append(notelst, tmp);     

      g_free(data);
      morerow = FALSE;
      break;

    case GEO:
      /* Geo Location (latitude, longitude) */
      /* load: long then latit 
         es. lati -0,3 long 5,4 ==> 5,4,-0,3 */
      tmp = g_strdup_printf("Geo: %s\n", data);
      notelen += strlen(tmp);
      notelst = g_list_append(notelst, tmp);      

      g_free(data);
      morerow = FALSE;
      break;
      
    case TITLE:
      /* titolo nell'organizzazione (professione) */
      r->professione = g_strdup(data);

      g_free(data);
      morerow = FALSE;
      break;

    case ORG:
      /* nome dell'organizzazione */
      tmp = g_strdup_printf(_("Org. Name: %s"),data);
      notelen += strlen(tmp);
      notelst = g_list_append(notelst, tmp);

      g_free(data);
      morerow = FALSE;
      break;

    case CATEGORIES:
      break;

    case NOTE:
      datalen = strlen(data);
      if (data[datalen-1] == '=')	
	morerow = TRUE; /* notes are on two or more lines */	 	
      
      tmp = g_strdup_printf(_("Other notes: %s"),data);     
      gcnotelen +=strlen(tmp);
      gcnotelst = g_list_append(gcnotelst, tmp);    

      g_free(data);
      break;
      
    case KEY:
      /* public key */
      switch (subtoken)
	{
	case PGP:
	  tmp = g_strdup_printf("Public Key (PGP):\n%s", data);	  
	  break;
	  
	case X509:
	  tmp = g_strdup_printf("Public Key (X509):\n%s", data);	  
	  break;

	default:
	  tmp = g_strdup_printf("Key:\n%s",data);
	  break;
	} /* switch subtoken */     
      notelen += strlen(tmp);
      notelst = g_list_append(notelst, tmp);

      g_free(data);
      morerow = FALSE;
      break;

    case END:
      /* collegare le liste mail e tel */
      r->mail_list = mail;
      r->tel_list = tel;
      r->nmail = nmail;
      r->ntel = ntel;
      r->note = rubrica_import_gcard_merge_note(notelst, gcnotelst, 
						notelen, gcnotelen);

      if (gcnotelst)
	rubrica_import_gcard_free_note(gcnotelst);
      if (notelst)
	rubrica_import_gcard_free_note(notelst);

      /* inserire l'elemento nella lista rubrica */
      rubrica_inserisci_elemento(r);
      rubrica_view_inc_indirizzi();

      /* impostare il flag di lista vuota a falso */
      /* visualizzo la nuova scheda in lista */
      rubrica_inserisci_scheda(r);   
   
      if (data)
	g_free(data);
      morerow = FALSE;
      break;

      
    case BAD:      
      if (morerow)
	{
	  tmp = g_strdup(data);     
	  gcnotelen +=strlen(tmp);
	  gcnotelst = g_list_append(gcnotelst, tmp);    
	}
      break;
      
      default:
      break;
    }
}

TokenGCard rubrica_import_gcard_decode_token(gchar *str, SubGToken *st)
{
  gchar *tmp;
  gboolean decoded = FALSE;
  gint i=0, size = 0;

  size = sizeof(gtoken) / sizeof (gtoken[0]);

  tmp = strtok(str, ";");

  while (!decoded && i < size)
    {
      if ((strcmp(tmp, gtoken[i].str)) == 0)
	{
	  if(gtoken[i].subtok)
	    {
	      tmp = strtok(NULL, "");

	      *st = rubrica_import_gcard_decode_subtoken(tmp);
	    }

	  decoded = TRUE;
	  return gtoken[i].tok;
	}
      else
	i++;
    }

  return (TokenGCard) BAD;
}

SubGToken rubrica_import_gcard_decode_subtoken(gchar *str)
{
  gchar *alias;
  gboolean decoded = FALSE;
  gint i=0, size = 0;
  
  size = sizeof(subgtoken) / sizeof (subgtoken[0]);
  alias = str;

  while (!decoded && i < size)
    {
      if ((strcmp(str, subgtoken[i].str) == 0))	
	decoded = TRUE; 	  	 
      else
	i++;
    }

  if (decoded)
    return (SubGToken) subgtoken[i].stok;
  
  return (SubGToken) SUB_BAD;;
}


void rubrica_import_gcard_decode_string(char *buf, gchar *dec[7])
{
  gint i, j, k;
  gchar *tmp;

  /* inizializzazione buffer decodificato */
  for(i=0; i<8;i++)
    dec[i]=NULL;

  tmp = g_malloc (strlen(buf));
  i = 0;
  j = 0;
  k = 0;
  while (buf[j] != '\0')
    {
      if(buf[j] != ';')
	{
	  tmp[k] = buf[j];
	  j++;
	  k++;
	}	 
      else
	{
	  tmp[k]='\0';
	  dec[i] = g_strdup(tmp);
	  i++;
	  j++;
	  k=0;
	}		
    }
  tmp[k]='\0';
  dec[i] = g_strdup(tmp); 
}


gchar *rubrica_import_gcard_decode_note(GList *l, gchar *st)
{
  gchar *tmp;

  g_return_val_if_fail(l != NULL, NULL);
  
  while(l->next)
    {
      if (l->data)
	tmp = (gchar*) l->data;     
      else 
	tmp = "";

      strncat(st, tmp, strlen(tmp));
      l = g_list_next(l);
    }   

  return (gchar *) st;
}

gchar *rubrica_import_gcard_merge_note(GList *l1, GList *l2, gint len1, gint len2)
{
  gchar *str1=NULL, *str2=NULL, *str3=NULL;
  gint len;

  len = len1 + len2 + 2;

  if (len1 > 0)
    {
      str1 = (gchar*) g_malloc((sizeof(gchar)*len1));
      str1[0]='\0';
      rubrica_import_gcard_decode_note(l1, str1);
    }
  if (len2 > 0)
    {
      str2 = (gchar*) g_malloc((sizeof(gchar)*len2));
      str2[0]='\0';
      rubrica_import_gcard_decode_note(l2, str2);
    }

  str3 = (gchar*) g_malloc(sizeof(gchar)*len);
  str3[0]='\0';
  if (str1 && str2)
    {
      strcpy(str3, str1);
      strcat(strcat(str3,"\n"),str2);
    }
  else 
    if (str1)      
      strcpy(str3, str1);      
  else
    if (str2)
      strcpy(str3, str2);  
  else
    strcpy(str3,"\n");
  
  g_free(str1);
  g_free(str2);

  return (gchar*) str3;
}

void rubrica_import_gcard_free_note(GList *n)
{
  GList *nodo;
  gint i;

  g_return_if_fail( n != NULL);

  i = g_list_length(n);

  while (i)
    {
      nodo = g_list_last(n);
      n = g_list_remove_link(n, nodo);
      
      if (nodo->data)
	  g_free(nodo->data);

      if (nodo)
	g_list_free_1(nodo);      
      i--;
    } 
  n = NULL;
}


void rubrica_import_gcard_split_buffer(gchar *buf, gchar **s, gchar **dt, gboolean *fl)
{
  gchar *tmp1, *tmp2;

  g_return_if_fail( buf != NULL);

  tmp1 = g_strdup(buf);
  tmp2 = g_strdup(buf);

  strtok(tmp1, ":");
  if (tmp1)
    *s = g_strdup(tmp1);

  if ((strcmp(tmp1, "REV") == 0) ||
      (strcmp(tmp1, "TZ") == 0 ) ||
      (strcmp(tmp1, "URL") == 0))
    {
      gint i;

      i = strlen(tmp1)+1;
      
      while(i--)
	tmp2++;

      rubrica_import_gcard_clean_buffer(tmp2, fl);
      *dt = g_strdup(tmp2);
    }
  else
    {
      g_strreverse(tmp2);
      strtok(tmp2, ":");
      g_strreverse(tmp2);
      rubrica_import_gcard_clean_buffer(tmp2, fl);
      
      if (tmp2)
	*dt = g_strdup(tmp2);
      else 
	*dt='\0';
    }
}


void rubrica_import_gcard_clean_buffer(gchar *buffer, gboolean *fl)
{
  int i=0;

  g_return_if_fail( buffer != NULL);

  while(buffer[i])
    {
      if((buffer[i] == '\r') && (buffer[i+1] == '\n'))
	{
	  if (buffer[i-1] == '=')
	    {
	      *fl = TRUE;  
	      buffer[i-1] = ' ';
	    }
	  buffer[i] = ' ';
	  buffer[i+1] = '\0';	  
	}
      if ((buffer[i] == '=') && (buffer[i+1] == 'E') && (buffer[i+2] == '8'))
	{
	  buffer[i]='';
	  buffer[i+1] = buffer [i+2] = ' ';
	}
      /* TODO: testare con le altre lettere accentate */ 
      else 
	i++;
    }
}





